@@ -3,7 +3,7 @@
 *           D e v i c e   C o n t e x t   F o r   P r i n t i n g               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDCPrint.cpp,v 1.47 2005/01/16 16:06:06 fox Exp $                       *
+* $Id: FXDCPrint.cpp,v 1.54 2006/01/22 17:58:21 fox Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -139,10 +139,18 @@ void FXDCPrint::bbox(FXfloat x,FXfloat y){
 
 // Send the range of coordinates that will be sent
 FXbool FXDCPrint::setContentRange(FXint pxminArg, FXint pyminArg, FXint pxmaxArg, FXint pymaxArg){
-  pxmin=pxminArg;
-  pymin=pyminArg;
-  pxmax=pxmaxArg;
-  pymax=pymaxArg;
+  if(flags&PRINT_LANDSCAPE){
+    pxmin=pyminArg;
+    pymin=pxminArg;
+    pxmax=pymaxArg;
+    pymax=pxmaxArg;
+    }
+  else{
+    pxmin=pxminArg;
+    pymin=pyminArg;
+    pxmax=pxmaxArg;
+    pymax=pymaxArg;
+    }
   return TRUE;    // Should we check for appropriate ranges?
   }
 
@@ -166,8 +174,10 @@ void FXDCPrint::tfm(FXfloat& xo,FXfloat& yo,FXfloat xi,FXfloat yi){
   if(flags&PRINT_LANDSCAPE){
     mxmin=static_cast<FXfloat>(mediabb.ymin);
     mxmax=static_cast<FXfloat>(mediabb.ymax);
-    mymin=static_cast<FXfloat>(mediawidth-mediabb.xmax);
-    mymax=static_cast<FXfloat>(mediawidth-mediabb.xmin);
+    //mymin=static_cast<FXfloat>(mediawidth-mediabb.xmax);
+    //mymax=static_cast<FXfloat>(mediawidth-mediabb.xmin);
+    mymin=static_cast<FXfloat>(mediabb.xmin);
+    mymax=static_cast<FXfloat>(mediabb.xmax);
     mxrange=mxmax-mxmin;
     myrange=mymax-mymin;
     //xo=xi;
@@ -197,7 +207,7 @@ void FXDCPrint::tfm(FXfloat& xo,FXfloat& yo,FXfloat xi,FXfloat yi){
 FXbool FXDCPrint::beginPrint(FXPrinter& job){
   int numpages;
 
-  Yr=480;  // This is essentially the height of the page(used so that the upper left hand corner is the origin)
+  Yr=792;  //480 // This is essentially the height of the page(used so that the upper left hand corner is the origin)
   Xr=0;
 
   // Print to file
@@ -264,6 +274,7 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
     docbb.ymax=(FXfloat)(job.mediaheight-job.topmargin);
     outf("%%%%BoundingBox: %d %d %d %d\n",(int)docbb.xmin,(int)docbb.ymin,(int)docbb.xmax,(int)docbb.ymax);
     }
+  setContentRange((int)docbb.xmin, (int)docbb.ymin, (int)docbb.xmax, (int)docbb.ymax);
 
   // Calculate number of pages
   numpages=0;
@@ -779,8 +790,7 @@ void FXDCPrint::fillComplexPolygonRel(const FXPoint*,FXuint){
   }
 
 
-// Draw string (only foreground bits)
-// Contributed by S. Ancelot <sancelot@online.fr>
+// Draw string with base line starting at x, y
 void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
 /*
   FXfloat xx,yy;
@@ -806,15 +816,17 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
   // old font size was hardcoded...
   //outf("(%s) %g %g %d /Courier drawText\n",string,xx,yy,15);
 
-  FXfloat pxrange=static_cast<FXfloat>(pxmax-pxmin);
-  FXfloat pyrange=static_cast<FXfloat>(pymax-pymin);
-  FXfloat mxmin,mxmax,mymin,mymax,mxrange,myrange;
+  //FXfloat pxrange=static_cast<FXfloat>(pxmax-pxmin);
+  //FXfloat pyrange=static_cast<FXfloat>(pymax-pymin);
+  //FXfloat mxmin,mxmax,mymin,mymax,mxrange,myrange;
 
-  if(flags&PRINT_LANDSCAPE){
+/*  if(flags&PRINT_LANDSCAPE){
     mxmin=static_cast<FXfloat>(mediabb.ymin);
     mxmax=static_cast<FXfloat>(mediabb.ymax);
-    mymin=static_cast<FXfloat>(mediawidth-mediabb.xmax);
-    mymax=static_cast<FXfloat>(mediawidth-mediabb.xmin);
+    //mymin=static_cast<FXfloat>(mediawidth-mediabb.xmax);
+    //mymax=static_cast<FXfloat>(mediawidth-mediabb.xmin);
+    mymin=static_cast<FXfloat>(mediabb.xmin);
+    mymax=static_cast<FXfloat>(mediabb.xmax);
     mxrange=mxmax-mxmin;
     myrange=mymax-mymin;
     }
@@ -826,13 +838,15 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
     mxrange=mxmax-mxmin;
     myrange=mymax-mymin;
     }
+*/
   FXfloat fsize=0.1f*font->getSize();
   // Hack...
   // Account for dpi and scale up or down with graph...
   // Perhaps override screen resolution via registry
-  FXint screenres=getApp()->reg().readUnsignedEntry("SETTINGS","screenres",100);
+  //  FXint screenres=getApp()->reg().readUnsignedEntry("SETTINGS","screenres",100);
+
   // Validate
-  if(screenres<50) screenres=50;
+/*  if(screenres<50) screenres=50;
   if(screenres>200) screenres=200;
 
   if(pyrange/pxrange<=myrange/mxrange){ // short/wide
@@ -841,7 +855,7 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
   else{// tall/thin
     fsize *= (myrange/pyrange)*(screenres/72.f);
     }
-
+*/
 
   FXString fname=font->getName();
   if(fname=="times"){
@@ -856,11 +870,11 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
   else{
     fname="Courier";
     }
-  if(font->getWeight()==FONTWEIGHT_BOLD){
-    if(font->getSlant()==FONTSLANT_ITALIC){
+  if(font->getWeight()==FXFont::Bold){
+    if(font->getSlant()==FXFont::Italic){
       fname+="-BoldItalic";
       }
-    else if(font->getSlant()==FONTSLANT_OBLIQUE){
+    else if(font->getSlant()==FXFont::Oblique){
       fname+="-BoldOblique";
       }
     else {
@@ -868,10 +882,10 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
       }
     }
   else{
-    if(font->getSlant()==FONTSLANT_ITALIC){
+    if(font->getSlant()==FXFont::Italic){
       fname+="-Italic";
       }
-    else if(font->getSlant()==FONTSLANT_OBLIQUE){
+    else if(font->getSlant()==FXFont::Oblique){
       fname+="-Oblique";
       }
     }
@@ -879,22 +893,37 @@ void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
     fname+="-Roman";
     }
 
-
-  //outf("(%s) %g %g %d /Courier drawText\n",string,xx,yy,(int)fsize);
   outf("(%s) %g %g %d /%s drawText\n",string,xx,yy,(int)fsize,fname.text());
   }
 
 
-// Draw string (both foreground and background bits)
+// Draw string with base line starting at x, y
+void FXDCPrint::drawText(FXint x,FXint y,const FXString& string){
+  drawText(x,y,string.text(),string.length());
+  }
+
+
+// Draw string with base line starting at x, y over filled background
 void FXDCPrint::drawImageText(FXint,FXint,const FXchar*,FXuint){
   }
 
 
+// Draw string with base line starting at x, y over filled background
+void FXDCPrint::drawImageText(FXint x,FXint y,const FXString& string){
+  drawImageText(x,y,string.text(),string.length());
+  }
+
+
 // Draw area from source
 void FXDCPrint::drawArea(const FXDrawable*,FXint,FXint,FXint,FXint,FXint,FXint){
   }
 
 
+// Draw area stretched area from source
+void FXDCPrint::drawArea(const FXDrawable*,FXint,FXint,FXint,FXint,FXint,FXint,FXint,FXint){
+  }
+
+
 // Draw image
 // Contibuted by dwalz@cs.uni-magdeburg.de
 void FXDCPrint::drawImage(const FXImage *img,FXint dx,FXint dy){
