@@ -3,7 +3,7 @@
 *                           O p e n G L   O b j e c t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLObject.cpp,v 1.33 2005/01/16 16:06:07 fox Exp $                      *
+* $Id: FXGLObject.cpp,v 1.37.2.3 2006/07/28 00:56:30 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -63,12 +63,11 @@ using namespace FX;
 
 namespace FX {
 
+
 // Object implementation
 FXIMPLEMENT(FXGLObject,FXObject,NULL,0)
 
 
-
-
 // Get bounding box
 void FXGLObject::bounds(FXRangef& box){
   box.upper.x=box.lower.x=0.0f;
@@ -307,27 +306,6 @@ void FXGLLine::draw(FXGLViewer*){
   glVertex3fv(fm.pos);
   glVertex3fv(to.pos);
   glEnd();
-
-/*
-static GLint list=-1;
-
-FXFont *font=FXApp::instance()->getNormalFont();
-
-int first,last,count;
-if(list<0){
-first=font->getMinChar();
-last=font->getMaxChar();
-count=last-first+1;
-list=glGenLists(count);
-glUseFXFont(font,first,count,list);
-FXTRACE((1,"first=%d last=%d list=%d\n",first,last,list));
-}
-
-  glColor3f(1.0,0.0,0.0);
-  glRasterPos2f(0,0);
-  glListBase(list);
-  glCallLists(26,GL_UNSIGNED_BYTE,(GLubyte*)"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
-*/
 #endif
   }
 
